@@ -55,13 +55,14 @@
 #define ANZWARN		13
 
 #define programname	"xa"
-#define progversion	"v2.3.9"
+#define progversion	"v2.3.10"
 #define authors		"Written by Andre Fachat, Jolse Maginnis, David Weinehall and Cameron Kaiser"
 #define copyright	"Copyright (C) 1989-2019 Andre Fachat, Jolse Maginnis, David Weinehall\nand Cameron Kaiser."
 
 /* exported globals */
 int ncmos, cmosfl, w65816, n65816;
 int masm = 0;
+int ppinstr = 0;
 int nolink = 0;
 int romable = 0;
 int romaddr = 0;
@@ -210,6 +211,10 @@ int main(int argc,char *argv[])
 	  case 'M':
 		masm = 1;	/* MASM compatibility mode */
 		break;
+          case 'S':
+                ppinstr = 1;    /* preprocessor substitution in strings ok */
+                fprintf(stderr, "Warning: -S is deprecated and will be removed in 2.4+!\n");
+		break;
 	  case 'O':		/* output charset */
 		{
 		  char *name = NULL;
@@ -267,7 +272,7 @@ int main(int argc,char *argv[])
 		break;
 	  case 'x':		/* old filename behaviour */
 		oldfile = 1;
-		fprintf(stderr, "Warning: -x is now deprecated and may disappear in future versions!\n");
+		fprintf(stderr, "Warning: -x is now deprecated and will be removed in 2.4+!\n");
 		break;
 	  case 'I':
 		if(argv[i][2]==0) {
@@ -392,6 +397,8 @@ int main(int argc,char *argv[])
 		 r_mode(RMODE_RELOC);
 		 segment = SEG_TEXT;
 	       } else {
+		 /* prime old_segment in r_mode with SEG_TEXT */
+	         segment = SEG_ABS;
 		 r_mode(RMODE_ABS);
 	       }
 
@@ -460,11 +467,13 @@ int main(int argc,char *argv[])
 
 		    seg_pass2();
 
-	            if(!relmode) {
-	              r_mode(RMODE_ABS);
+	     	    if(relmode) {
+		 	r_mode(RMODE_RELOC);
+		 	segment = SEG_TEXT;
 	            } else {
-	              r_mode(RMODE_RELOC);
-		      segment = SEG_TEXT;
+		 	/* prime old_segment in r_mode with SEG_TEXT */
+	         	segment = SEG_ABS;
+		 	r_mode(RMODE_ABS);
 	            }
                     er=pass2();
                } 
@@ -825,8 +834,6 @@ static void usage(int default816, FILE *fp)
             programname);
 	fprintf(fp,
 	    " -v           verbose output\n"
-	    " -x           old filename behaviour (overrides `-o', `-e', `-l')\n"
-	    "              This is deprecated and may disappear in future versions!\n"
             " -C           no CMOS-opcodes\n"
             " -W           no 65816-opcodes%s\n"
             " -w           allow 65816-opcodes%s\n",
@@ -854,11 +861,16 @@ static void usage(int default816, FILE *fp)
 	    "                Other segments must be specified with `-b?'\n"
 	    " -G           suppress list of exported globals\n");
 	fprintf(fp,
+	    " -p?          set preprocessor character to ?, default is #\n"
 	    " -DDEF=TEXT   defines a preprocessor replacement\n"
 	    " -Ocharset    set output charset (PETSCII, ASCII, etc.), case-sensitive\n"
 	    " -Idir        add directory `dir' to include path (before XAINPUT)\n"
 	    "  --version   output version information and exit\n"
 	    "  --help      display this help and exit\n");
+	fprintf(fp,
+	    "== These options are deprecated and will be removed in 2.4+! ==\n"
+	    " -x           old filename behaviour (overrides `-o', `-e', `-l')\n"
+	    " -S           allow preprocessor substitution within strings\n");
 }
 
 /*
