@@ -479,48 +479,44 @@ printf(" wrote %02x %02x %02x %02x %02x %02x\n",
 	       dsb_len = 0;
           } else
 	  if(n==Ktext) {
-/*	    if(segment!=SEG_ABS) {    */
 	      segment = relmode ? SEG_TEXT : SEG_ABS;
 	      t[0]=Ksegment;
 	      t[1]=segment;
 	      *ll=2;
               er=E_OKDEF;
-/*	    } else {
-	      er=E_ILLSEGMENT;
-	    }                        */
 	  } else
 	  if(n==Kdata) {
-/*  	    if(segment!=SEG_ABS) {   */
+  	    if(relmode) {   
 	      segment = SEG_DATA;
 	      t[0]=Ksegment;
 	      t[1]=SEG_DATA;
 	      *ll=2;
               er=E_OKDEF;
-/*	    } else {
+	    } else {
 	      er=E_ILLSEGMENT;
-	    }                        */
+	    } 
 	  } else
 	  if(n==Kbss) {
-/*  	    if(segment!=SEG_ABS) {   */
+  	    if(relmode) { 
 	      segment = SEG_BSS;
 	      t[0]=Ksegment;
 	      t[1]=SEG_BSS;
 	      *ll=2;
               er=E_OKDEF;
-/*	    } else {
+	    } else {
 	      er=E_ILLSEGMENT;
-	    }                        */
+	    } 
 	  } else
 	  if(n==Kzero) {
-/*  	    if(segment!=SEG_ABS) {   */
+  	    if(relmode) {   
 	      segment = SEG_ZERO;
 	      t[0]=Ksegment;
 	      t[1]=SEG_ZERO;
 	      *ll=2;
               er=E_OKDEF;
-/*	    } else {
+	    } else {
 	      er=E_ILLSEGMENT;
-	    }                        */
+	    }  
 	  } else
 	if (n==Kbin) {
 		int j;
@@ -704,8 +700,14 @@ fprintf(stderr, "E_NODEF pass1 xat.c\n");
                } else
                     sy=4+nk;	/* absolute or zero page */
 
-		/* length counter set to maximum length + 1 */
-               bl=Maxbyt+1;
+	       /* length counter set to maximum length + 1 */
+	       if (w65816 || (t[l-1]=='@' || t[l-1] == '!')) {
+		       	/* for 65816 allow addressing modes up to 4 byte overall length */
+               		bl=Maxbyt+1;
+	       } else {
+		       	/* for other modes only check for addressing modes up to 3 byte overall length */
+		 	bl=Maxbyt;
+	       }
                
 		/* find best fit for length of this operand */
                while(--bl)
@@ -1375,9 +1377,19 @@ fprintf(stderr, "Kdsb E_DSB %i\n", j);
                          }
                     }
                }
-                
-               bl=Maxbyt+1;
                
+	       /* set bl to maximum overall length +1 as while() below starts with decrementing it */
+	       if (w65816 || (t[*ll-1]=='@' || t[*ll-1] == '!')) {
+		       	/* for 65816 allow addressing modes up to 4 byte overall length */
+               		bl=Maxbyt+1;
+	       } else {
+		       	/* for other modes only check for addressing modes up to 3 byte overall length */
+		 	bl=Maxbyt;
+	       }
+              
+#ifdef DEBUG_AM
+	      printf("--- trying to find am using: (max+1) bl=%d, sy=%d\n", bl, sy); 
+#endif
                while(--bl)
                {
                     if((am=at[sy][bl-1])>=0)
@@ -1415,8 +1427,8 @@ fprintf(stderr, "Kdsb E_DSB %i\n", j);
                {
 #ifdef DEBUG_AM
 fprintf(stderr,
-"b4: pc= %d, am = %d and vv[0] = %d, optimize = %d, bitmask = %d\n",
-	pc[segment], am, vv[0], fl, (vv[0]&0xffff00));
+"b4: pc= %d, am = %d and vv[0] = %d, optimize = %d, bitmask = %u, er=%d\n",
+	pc[segment], am, vv[0], fl, (vv[0]&0xffff00), er);
 #endif
 
 /* terrible KLUDGE!!!! OH NOES!!!1!
@@ -1455,15 +1467,21 @@ fprintf(stderr,
                else
                {
                     bl=le[am];
+		    if ((am != 11 && am != 16) && (vv[0] > 255 || vv[0] < -256) && bl == 2) {
+			    er = E_OVERFLOW;
+		    } else
+		    if ((am != 11 && am != 16) && (vv[0] > 65535 || vv[0] < -65536) && (bl == 2 || bl == 3)) {
+			    er = E_OVERFLOW;
+		    } else
                     if( ((ct[n][am]&0x400) && memode) || ((ct[n][am]&0x800) && xmode)) {
                          bl++;
-		}
+		    }
                     *ll=bl;
 
                }
 
 #ifdef DEBUG_AM
-fprintf(stderr, "byte length is now %d\n", bl);
+fprintf(stderr, "byte length is now %d, am=%d, er=%d\n", bl, am, er);
 #endif
 
                if(!er)
@@ -1512,6 +1530,7 @@ fprintf(stderr, "address mode: %i address: %i\n", am, vv[0]);
                               }
                          } else
                          if(am==11 || am==16) {
+			   /* relative, relative long */
 			   if((segment!=SEG_ABS) && (!rlt[0])) {
 			     er=E_ILLPOINTER;
 			   } else {
